package com.ningdatech.pmapi.user.manage;

import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.common.util.StrUtils;
import com.ningdatech.pmapi.user.convert.UserInfoConvertor;
import com.ningdatech.pmapi.user.security.auth.credential.CredentialAuthToken;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import lombok.AllArgsConstructor;
import org.springframework.security.core.context.SecurityContextImpl;
import org.springframework.session.Session;
import org.springframework.session.SessionRepository;
import org.springframework.session.data.redis.RedisIndexedSessionRepository;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * UserAuthHelper
 * </p>
 *
 * @author WendyYang
 * @since 2023/5/5
 **/
@Component
@AllArgsConstructor
public class UserAuthManage {

    private final UserInfoHelper userInfoHelper;
    private final RedisIndexedSessionRepository redisSessionRepository;

    @SuppressWarnings({"rawtypes", "unchecked"})
    public void refreshSession(Long userId) {
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        List<String> sessionIds = getSessionIds(userFullInfo);
        if (sessionIds.isEmpty()) {
            return;
        }
        SessionRepository sessionRepository = redisSessionRepository;
        UserInfoDetails details = UserInfoConvertor.toUserInfoDetails(userFullInfo);
        CredentialAuthToken cat = new CredentialAuthToken(details, details.getPassword(), details.getAuthorities());
        sessionIds.forEach(sessionId -> {
            Session session = redisSessionRepository.findById(sessionId);
            SecurityContextImpl context = session.getAttribute("SPRING_SECURITY_CONTEXT");
            context.setAuthentication(cat);
            session.setAttribute("SPRING_SECURITY_CONTEXT", context);
            sessionRepository.save(session);
        });
    }

    private List<String> getSessionIds(UserFullInfoDTO userFullInfo) {
        List<String> sessionIds = new ArrayList<>();
        String identifier = userFullInfo.getIdentifier();
        if (StrUtils.isNotBlank(identifier)) {
            sessionIds.addAll(redisSessionRepository.findByPrincipalName(identifier).keySet());
        }
        String mobile = userFullInfo.getMobile();
        if (StrUtils.isNotBlank(mobile)) {
            sessionIds.addAll(redisSessionRepository.findByPrincipalName(mobile).keySet());
        }
        Long accountId = userFullInfo.getAccountId();
        if (accountId != null) {
            sessionIds.addAll(redisSessionRepository.findByPrincipalName(accountId.toString()).keySet());
        }
        return sessionIds;
    }

    public void kickOff(Long userId) {
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);
        List<String> sessionIds = getSessionIds(userFullInfo);
        sessionIds.forEach(redisSessionRepository::deleteById);
    }

}
